Update

Enum Update 

pub enum Update<'g, K, V, S>
where K: Key, V: Value + 'g, S: Smr<K, V> + 'g,
{ Absent { new: Option<V>, }, Success(Updated<'g, K, V, S>), Break { old: Shared<'g, K, V, S>, new: Option<V>, }, }
Outcome of a call to ConcurrentMap::update_with.

Variants



Absent

Key was not present.

Fields

new: Option<V>

Latest value passed as argument or returned from closure.



Success(Updated<'g, K, V, S>)

Value was successfully updated.



Break

Fields

old: Shared<'g, K, V, S>

Latest value observed by closure.

new: Option<V>

Latest value passed as argument or returned from closure.