pub struct Seize(/* private fields */);Expand description
seize::Collector backend for safe memory reclamation.
Defaults to a batch size of 256, which we found to provide the best balance of throughput and reclamation efficiency in our benchmarks.
Examples
use arctic::ConcurrentMap;
use arctic::concurrent::smr::Seize;
let map = ConcurrentMap::<u64, Box<u64>, Seize>::with_smr(Seize::from(
seize::Collector::new().batch_size(256)
));Trait Implementations
Sourceimpl<K: Key, V: Value> Smr<K, V> for Seize
impl<K: Key, V: Value> Smr<K, V> for Seize
Sourcetype Guard<'g> = LocalGuard<'g>
where
V: 'g,
Self: 'g
type Guard<'g> = LocalGuard<'g> where V: 'g, Self: 'g
Guard type that protects nodes and values during its lifetime,
can be used to retire allocations, and unprotects when it is dropped.