pub struct Owned<G: Guard<V>, V: Value> { /* private fields */ }Expand description
Guard that provides read-only access to a removed value while preventing the value from being freed. Retires the value on drop.
Note: this value may still be concurrently accessed by other threads, so this guard cannot safely provide mutable access.