Updated

Struct Updated 

pub struct Updated<G: Guard<V>, V: Value> { /* private fields */ }
Guard that provides read-only access to both the old and new values of an atomic update operation, preventing both from being freed.

Retires the old value on drop.

Implementations

Source

impl<G, V> Updated<G, V>
where G: Guard<V>, V: Value,

pub fn old(&self) -> &V::Borrowed

Return the old value before updating.

pub fn new(&self) -> &V::Borrowed

Return the new value after updating.