Error

Enum Error 

pub enum Error {
    NoBleAdaptor,
    NoControlPoint,
    NoDevice,
    NotConnected,
    NoDataType,
    CharacteristicNotFound,
    InvalidData,
    InvalidLength,
    NullCommand,
    WrongResponse,
    WrongType,
    BleError(Error),
}
Error type for general errors and Ble errors from btleplug

Variants



NoBleAdaptor

Not bluetooth adapter found when trying to scan



NoControlPoint

Could not create control point link



NoDevice

Could not find a device when trying to connect



NotConnected

Device is not connected, but function was called that requires it



NoDataType

No measurement type selected



CharacteristicNotFound

Device is missing a characteristic that was used



InvalidData

Data packets received from device could not be parsed



InvalidLength

Not enough data was received



NullCommand

Command to write to PMD control point is Null



WrongResponse

Tried to create a struct using the wrong control point response



WrongType

Tried to set a setting using with a H10MeasurementType that doesn’t support that feature



BleError(Error)

An error occurred in the underlying BLE library