pub enum NotifyStream {
Battery,
HeartRate,
MeasurementCP,
MeasurementData,
}Expand description
A list of stream types that can be subscribed to
Variants
Battery
Receive battery updates
HeartRate
Receive heart rate updates
MeasurementCP
Receive updates from the control points, only for use within the library
MeasurementData
Receive updates from the PMD data stream (acceleration or ecg)