pub struct BoxedSlice<I, R: ?Sized = [u8]> { /* private fields */ }Expand description
An owned, dynamically sized key that satisfies an Invariant.
Implementations
Sourceimpl<I, R> BoxedSlice<I, R>
impl<I, R> BoxedSlice<I, R>
Sourceimpl<I, R: ?Sized> BoxedSlice<I, R>
impl<I, R: ?Sized> BoxedSlice<I, R>
Sourcepub const unsafe fn new_unchecked(key: Box<R>) -> Self
pub const unsafe fn new_unchecked(key: Box<R>) -> Self
Construct a boxed slice without validating.
SAFETY
Caller must guarantee that raw satisfies the invariant, i.e.,
I::validate(key) would return Ok(()).
Sourcepub const fn as_slice(&self) -> &Slice<I, R>
pub const fn as_slice(&self) -> &Slice<I, R>
Get a borrowed Slice that preserves the invariant.
Sourcepub fn into_boxed_slice(self) -> Box<R>
pub fn into_boxed_slice(self) -> Box<R>
Get an owned boxed slice.