Slice

Struct Slice 

pub struct Slice<I, R: ?Sized = [u8]> { /* private fields */ }
A borrowed, dynamically sized key that satisfies an Invariant.

Implementations

Source

impl<I, R> Slice<I, R>
where I: Invariant, R: ?Sized + Raw,

pub fn new(key: &R) -> Result<&Self, I::Error>

Construct a slice after validating.

Returns Ok if the input key satisfies the invariant.

Source

impl<I, R: ?Sized> Slice<I, R>

pub const unsafe fn new_unchecked(key: &R) -> &Self

Safety

Caller must ensure key upholds invariants, i.e., I::validate(key) would return Ok.

Source

impl<I> Slice<I, str>

pub const fn as_str(&self) -> &str

Get a reference to the underlying str.

Source

impl<I> Slice<I, [u8]>

pub const fn as_bytes(&self) -> &[u8]

Get a reference to the underlying [u8].