Module sequential

Module sequential 

Non-concurrent implementation of adaptive radix tree.

Modules

map
Auxiliary types for use with SequentialMap.
set
Auxiliary types for use with SequentialSet.

Structs

EntryIter
Iterator over keys and immutable references to values.
EntryIterMut
Iterator over keys and mutable references to values.
Map
Non-concurrent map that supports lexicographically ordered range and prefix scans.
Set
Non-concurrent set.
Shard
Immutable reference to a subtree rooted at a key prefix, optionally bounded by a key range.
ShardMut
Mutable reference to a subtree rooted at a key prefix, optionally bounded by a key range.
ValueIter
Iterator over references to values.
ValueIterMut
Iterator over mutable references to values.

Traits

Value
Values that can be safely stored in a SequentialMap.