Entry

Enum Entry 

pub enum Entry<'g, 'k, K, V>
where K: Key, V: Value + 'g,
{ Vacant(Vacant<'g, 'k, K, V>), Occupied(Occupied<'g, V>), }
A logical entry in a SequentialMap that may be vacant or occupied.

See: btree_map::Entry.

Variants



Vacant(Vacant<'g, 'k, K, V>)

A vacant entry.



Occupied(Occupied<'g, V>)

An occupied entry.

Implementations

Source

impl<'g, 'k, K: Key, V: Value + 'g> Entry<'g, 'k, K, V>

pub fn or_insert(self, default: V) -> &'g mut V

Insert default if there is no value associated with this entry, then return a mutable reference to the current value.

See: btree_map::Entry::or_insert.

pub fn or_insert_with<F: FnOnce() -> V>(self, default: F) -> &'g mut V

Like or_insert, but lazily evaluates default.

See: btree_map::Entry::or_insert_with.

pub fn and_modify<F>(self, modify: F) -> Self
where F: FnOnce(&mut V),

If there is a value associated with this entry, then apply modify to it.

See: btree_map::Entry::and_modify.

Source

impl<'g, 'k, K: Key, V: Value + Default + 'g> Entry<'g, 'k, K, V>