Occupied

Struct Occupied 

pub struct Occupied<'g, V: Value + 'g> { /* private fields */ }
An occupied entry in a SequentialMap.

Implementations

Source

impl<'g, V: Value> Occupied<'g, V>

pub fn get(&self) -> &V

Get an immutable reference to the value in this entry.

See: btree_map::OccupiedEntry::get.

pub fn get_mut(&mut self) -> &mut V

Get a mutable reference to the value in this entry.

See: btree_map::OccupiedEntry::get_mut.

pub fn update(&mut self, value: V) -> V

Update the value in this entry to value.

See: btree_map::OccupiedEntry::insert.

pub fn into_mut(self) -> &'g mut V

Convert this entry into a mutable reference, preserving the lifetime.

See: btree_map::OccupiedEntry::into_mut.