Shard

Struct Shard 

pub struct Shard<'g, 'k, K: Key, V, R> { /* private fields */ }
Immutable reference to a subtree rooted at a key prefix, optionally bounded by a key range.

Implementations

Source

impl<'g, 'k, K, V, R> Shard<'g, 'k, K, V, R>
where K: Key, V: Value, R: Range<K::Read<'k>>,

pub fn entries(&self, order: Order) -> EntryIter<'g, 'k, K, V, R>

Get an iterator over keys and immutable references to values in order.

pub fn values(&self, order: Order) -> ValueIter<'g, 'k, K, V, R>

Get an iterator over immutable references to values in order.