ShardMut

Struct ShardMut 

pub struct ShardMut<'g, 'k, K: Key, V, R>(/* private fields */);
Mutable reference to a subtree rooted at a key prefix, optionally bounded by a key range.

Implementations

Source

impl<'g, 'k, K, V, R> ShardMut<'g, 'k, K, V, R>
where K: Key, V: Value, R: Range<K::Read<'k>>,

pub fn entries_mut(&mut self, order: Order) -> EntryIterMut<'g, 'k, K, V, R>

Get an iterator over keys and mutable references to values in O order.

pub fn values_mut(&mut self, order: Order) -> ValueIterMut<'g, 'k, K, V, R>

Get an iterator over mutable references to values in O order.

Methods from Deref<Target = Shard<'g, 'k, K, V, R>>

pub fn entries(&self, order: Order) -> EntryIter<'g, 'k, K, V, R>

Get an iterator over keys and immutable references to values in order.

pub fn values(&self, order: Order) -> ValueIter<'g, 'k, K, V, R>

Get an iterator over immutable references to values in order.