Lock-free concurrent implementation of adaptive radix tree.
Modules
- map
- Auxiliary types for use with
ConcurrentMap. - smr
- Implementations of
Smr. - value
- Values that can safely be stored in a
ConcurrentMap, and referenced behind ansmr::Guard.
Structs
- Entry
Iter - Iterator over keys and references to values.
- Map
- Lock-free concurrent map that supports lexicographically ordered, non-linearizable range and prefix scans.
- Shard
- Immutable reference to a subtree rooted at a key prefix, optionally bounded by a key range.
- Value
Iter - Iterator over references to values.
Traits
- Smr
- Provides safe memory reclamation for the given key and value type.
- Value
- Values that can safely be stored in a
ConcurrentMap.