pub unsafe fn deallocate_node(node: NonZeroU64)Expand description
Deallocate a previously retired node.
Safety
Caller must guarantee there are no live references to node,
and that node was previously retired via Guard::retire_node.
pub unsafe fn deallocate_node(node: NonZeroU64)Deallocate a previously retired node.
Caller must guarantee there are no live references to node,
and that node was previously retired via Guard::retire_node.