Values that can safely be stored in a ConcurrentMap,
and referenced behind an smr::Guard.
Structs
- Arc
- Transparent wrapper for
Arc<T>smart pointer that can be borrowed as anArcRef<T>. - ArcRef
- Transparent wrapper for
Arc<T>pointee that can be safely cloned into anArc<T>viaToOwned. - Owned
- Guard that provides read-only access to a removed value while preventing the value from being freed. Retires the value on drop.
- Shared
- Guard that provides read-only access to a value while preventing the value from being freed.
- Updated
- Guard that provides read-only access to both the old and new values of an atomic update operation, preventing both from being freed.
- Upserted
- Guard that provides read-only access to both the old and new values of an atomic upsert operation, preventing both from being freed.
Traits
- Value
- Values that can safely be stored in a
ConcurrentMap.