pub struct Updated<G: Guard<V>, V: Value> { /* private fields */ }Expand description
Guard that provides read-only access to both the old and new values of an atomic update operation, preventing both from being freed.
Retires the old value on drop.
Implementations