Upserted

Struct Upserted 

pub struct Upserted<G: Guard<V>, V: Value> { /* private fields */ }
Guard that provides read-only access to both the old and new values of an atomic upsert operation, preventing both from being freed.

Retires the old value on drop, if it existed.

Implementations

Source

impl<G, V> Upserted<G, V>
where G: Guard<V>, V: Value,

pub fn old(&self) -> Option<&V::Borrowed>

Return the old value before upserting.

pub fn new(&self) -> &V::Borrowed

Return the new value after upserting.