BoxedSlice

Struct BoxedSlice 

pub struct BoxedSlice<I, R: ?Sized = [u8]> { /* private fields */ }
An owned, dynamically sized key that satisfies an Invariant.

Implementations

Source

impl<I, R> BoxedSlice<I, R>
where I: Invariant, R: ?Sized + Raw,

pub fn new(key: impl Into<Box<R>>) -> Result<Self, (Box<R>, I::Error)>

Construct a boxed slice after validating.

Returns Ok if the input boxed slice satisfies the invariant.

Source

impl<I, R: ?Sized> BoxedSlice<I, R>

pub const unsafe fn new_unchecked(key: Box<R>) -> Self

Construct a boxed slice without validating.

SAFETY

Caller must guarantee that raw satisfies the invariant, i.e., I::validate(key) would return Ok(()).

pub const fn as_slice(&self) -> &Slice<I, R>

Get a borrowed Slice that preserves the invariant.

pub fn into_boxed_slice(self) -> Box<R>

Get an owned boxed slice.