pub struct NonNull;Expand description
Invariant ZST indicating this key does not contain any null bytes.
Allows a null byte to be internally appended to each key, which guarantees the prefix property and does not change the lexicographic ordering of the key.