pub trait Split: Key {
// Required method
fn split_last<'k>(key: &'k Self::Borrowed) -> (Self::Read<'k>, u8);
}Expand description
Key types that can split off their last byte, which enables an efficient set implementation.
Required Methods
Sourcefn split_last<'k>(key: &'k Self::Borrowed) -> (Self::Read<'k>, u8)
fn split_last<'k>(key: &'k Self::Borrowed) -> (Self::Read<'k>, u8)
Split a key into a reader and the last byte.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".