Non-concurrent implementation of adaptive radix tree.
Modules
- map
- Auxiliary types for use with
SequentialMap. - set
- Auxiliary types for use with
SequentialSet.
Structs
- Entry
Iter - Iterator over keys and immutable references to values.
- Entry
Iter Mut - Iterator over keys and mutable references to values.
- Map
- Non-concurrent map that supports lexicographically ordered range and prefix scans.
- Set
- Non-concurrent set.
- Shard
- Immutable reference to a subtree rooted at a key prefix, optionally bounded by a key range.
- Shard
Mut - Mutable reference to a subtree rooted at a key prefix, optionally bounded by a key range.
- Value
Iter - Iterator over references to values.
- Value
Iter Mut - Iterator over mutable references to values.
Traits
- Value
- Values that can be safely stored in a
SequentialMap.