pub struct ShardMut<'g, 'k, K: Key, V, R>(/* private fields */);Expand description
Mutable reference to a subtree rooted at a key prefix, optionally bounded by a key range.
Implementations
Sourceimpl<'g, 'k, K, V, R> ShardMut<'g, 'k, K, V, R>
impl<'g, 'k, K, V, R> ShardMut<'g, 'k, K, V, R>
Sourcepub fn entries_mut(&mut self, order: Order) -> EntryIterMut<'g, 'k, K, V, R> ⓘ
pub fn entries_mut(&mut self, order: Order) -> EntryIterMut<'g, 'k, K, V, R> ⓘ
Get an iterator over keys and mutable references to values in O order.
Sourcepub fn values_mut(&mut self, order: Order) -> ValueIterMut<'g, 'k, K, V, R> ⓘ
pub fn values_mut(&mut self, order: Order) -> ValueIterMut<'g, 'k, K, V, R> ⓘ
Get an iterator over mutable references to values in O order.